@@ -24,27 +24,19 @@ from pickle import dump
 from signal import signal, SIGINT, SIGTERM
 from urllib.parse import urlparse
 
-
-try:
-    from lollypop.lastfm import LastFM, LibreFM
-except Exception as e:
-    print(e)
-    print("$ sudo pip3 install pylast")
-    LastFM = None
-
 from lollypop.utils import init_proxy_from_gnome, emit_signal
-from lollypop.utils import get_network_available
 from lollypop.application_actions import ApplicationActions
-from lollypop.utils_file import is_audio, is_pls, install_youtube_dl
-from lollypop.define import Type, LOLLYPOP_DATA_PATH, ScanType, StorageType
+from lollypop.utils_file import get_file_type, install_youtube_dl
+from lollypop.define import LOLLYPOP_DATA_PATH, ScanType, StorageType, FileType
 from lollypop.database import Database
 from lollypop.player import Player
 from lollypop.inhibitor import Inhibitor
 from lollypop.art import Art
 from lollypop.logger import Logger
-from lollypop.search_spotify import SpotifySearch
+from lollypop.ws_director import DirectorWebService
 from lollypop.sqlcursor import SqlCursor
 from lollypop.settings import Settings
+from lollypop.database_cache import CacheDatabase
 from lollypop.database_albums import AlbumsDatabase
 from lollypop.database_artists import ArtistsDatabase
 from lollypop.database_genres import GenresDatabase
@@ -53,11 +45,9 @@ from lollypop.notification import NotificationManager
 from lollypop.playlists import Playlists
 from lollypop.objects_track import Track
 from lollypop.objects_album import Album
-from lollypop.objects_radio import Radio
-from lollypop.radios import Radios
 from lollypop.helper_task import TaskHelper
 from lollypop.helper_art import ArtHelper
-from lollypop.collectionscanner import CollectionScanner
+from lollypop.collection_scanner import CollectionScanner
 
 
 class Application(Gtk.Application, ApplicationActions):
@@ -102,12 +92,10 @@ class Application(Gtk.Application, ApplicationActions):
                     GLib.setenv("SSL_CERT_FILE", path, True)
                     break
         self.cursors = {}
-        self.scrobblers = []
         self.debug = False
         self.shown_sidebar_tooltip = False
         self.__window = None
         self.__fs_window = None
-        self.__spotify_timeout_id = None
         settings = Gio.Settings.new("org.gnome.desktop.interface")
         self.animations = settings.get_value("enable-animations").get_boolean()
         GLib.set_application_name("Lollypop")
@@ -178,12 +166,12 @@ class Application(Gtk.Application, ApplicationActions):
         styleContext.add_provider_for_screen(screen, cssProvider,
                                              Gtk.STYLE_PROVIDER_PRIORITY_USER)
         self.db = Database()
+        self.cache = CacheDatabase()
         self.playlists = Playlists()
-        self.albums = AlbumsDatabase()
-        self.artists = ArtistsDatabase()
-        self.genres = GenresDatabase()
-        self.tracks = TracksDatabase()
-        self.radios = Radios()
+        self.albums = AlbumsDatabase(self.db)
+        self.artists = ArtistsDatabase(self.db)
+        self.genres = GenresDatabase(self.db)
+        self.tracks = TracksDatabase(self.db)
         self.player = Player()
         self.inhibitor = Inhibitor()
         self.scanner = CollectionScanner()
@@ -192,7 +180,8 @@ class Application(Gtk.Application, ApplicationActions):
         self.art_helper = ArtHelper()
         self.art = Art()
         self.art.update_art_size()
-        self.spotify = SpotifySearch()
+        self.ws_director = DirectorWebService()
+        self.ws_director.start()
         if not self.settings.get_value("disable-mpris"):
             from lollypop.mpris import MPRIS
             MPRIS(self)
@@ -209,31 +198,6 @@ class Application(Gtk.Application, ApplicationActions):
                 not monitor.get_network_metered() and\
                 self.settings.get_value("recent-youtube-dl"):
             self.task_helper.run(install_youtube_dl)
-        self.start_spotify()
-
-    def start_spotify(self):
-        """
-            Start spotify timeout and start a new populate
-        """
-        if Type.SUGGESTIONS not in\
-                self.settings.get_value("shown-album-lists"):
-            return
-        monitor = Gio.NetworkMonitor.get_default()
-        if not monitor.get_network_metered() and\
-                get_network_available("SPOTIFY") and\
-                self.__spotify_timeout_id is None:
-            self.spotify.start()
-            self.__spotify_timeout_id = GLib.timeout_add_seconds(
-                3600, self.spotify.start)
-
-    def stop_spotify(self):
-        """
-            Stop spotify timeout and stop current populate
-        """
-        if self.__spotify_timeout_id is not None:
-            self.spotify.stop()
-            GLib.source_remove(self.__spotify_timeout_id)
-            self.__spotify_timeout_id = None
 
     def do_startup(self):
         """
@@ -256,8 +220,7 @@ class Application(Gtk.Application, ApplicationActions):
         """
         self.__window.container.stop()
         self.__window.hide()
-        if self.spotify.is_running:
-            self.spotify.stop()
+        if not self.ws_director.stop():
             GLib.timeout_add(100, self.quit, vacuum)
             return
         if self.settings.get_value("save-state"):
@@ -266,8 +229,6 @@ class Application(Gtk.Application, ApplicationActions):
         if vacuum:
             self.__vacuum()
             self.art.clean_artwork()
-        for scrobbler in self.scrobblers:
-            scrobbler.save()
         Gio.Application.quit(self)
         if GLib.environ_getenv(GLib.get_environ(), "DEBUG_LEAK") is not None:
             import gc
@@ -276,20 +237,6 @@ class Application(Gtk.Application, ApplicationActions):
                 s = str(x)
                 print(type(x), "\n  ", s)
 
-    def load_listenbrainz(self):
-        """
-            Load listenbrainz support if needed
-        """
-        if self.settings.get_value("listenbrainz-user-token").get_string():
-            from lollypop.listenbrainz import ListenBrainz
-            for scrobbler in self.scrobblers:
-                if isinstance(scrobbler, ListenBrainz):
-                    return
-            listenbrainz = ListenBrainz()
-            self.scrobblers.append(listenbrainz)
-            self.settings.bind("listenbrainz-user-token", listenbrainz,
-                               "user_token", 0)
-
     def fullscreen(self):
         """
             Go fullscreen
@@ -305,6 +252,8 @@ class Application(Gtk.Application, ApplicationActions):
             self.__fs_window.show()
             self.__fs_window.connect("destroy", on_destroy)
             self.__window.hide()
+        else:
+            self.__fs_window.destroy()
 
     @property
     def proxy_host(self):
@@ -345,20 +294,6 @@ class Application(Gtk.Application, ApplicationActions):
         """
         return self.__fs_window is not None
 
-    @property
-    def lastfm(self):
-        """
-            Get lastfm provider from scrobbler
-            @return LastFM/None
-        """
-        if LastFM is None:
-            return None
-        from pylast import LastFMNetwork
-        for scrobbler in self.scrobblers:
-            if isinstance(scrobbler, LastFMNetwork):
-                return scrobbler
-        return None
-
     @property
     def main_window(self):
         """
@@ -426,13 +361,6 @@ class Application(Gtk.Application, ApplicationActions):
                  open(LOLLYPOP_DATA_PATH + "/player.bin", "wb"))
             dump(self.player.queue,
                  open(LOLLYPOP_DATA_PATH + "/queue.bin", "wb"))
-            # Save current playlist
-            if isinstance(self.player.current_track, Radio):
-                playlist_ids = [Type.RADIOS]
-            else:
-                playlist_ids = []
-            dump(playlist_ids,
-                 open(LOLLYPOP_DATA_PATH + "/playlist_ids.bin", "wb"))
             if self.player.current_track.id is not None:
                 position = self.player.position
             else:
@@ -455,10 +383,9 @@ class Application(Gtk.Application, ApplicationActions):
             self.albums.clean(False)
             self.artists.clean(False)
             self.genres.clean(False)
-            SqlCursor.commit(self.db)
             SqlCursor.remove(self.db)
+            self.cache.clean(True)
 
-            from lollypop.radios import Radios
             with SqlCursor(self.db) as sql:
                 sql.isolation_level = None
                 sql.execute("VACUUM")
@@ -467,10 +394,6 @@ class Application(Gtk.Application, ApplicationActions):
                 sql.isolation_level = None
                 sql.execute("VACUUM")
                 sql.isolation_level = ""
-            with SqlCursor(Radios()) as sql:
-                sql.isolation_level = None
-                sql.execute("VACUUM")
-                sql.isolation_level = ""
         except Exception as e:
             Logger.error("Application::__vacuum(): %s" % e)
 
@@ -512,11 +435,6 @@ class Application(Gtk.Application, ApplicationActions):
             options = app_cmd_line.get_options_dict()
             if options.contains("debug"):
                 self.debug = True
-            # We are forced to enable scrobblers here if we want full debug
-            if not self.scrobblers:
-                if LastFM is not None:
-                    self.scrobblers = [LastFM(), LibreFM()]
-                self.load_listenbrainz()
             if options.contains("set-rating"):
                 value = options.lookup_value("set-rating").get_string()
                 try:
@@ -559,25 +477,24 @@ class Application(Gtk.Application, ApplicationActions):
                 audio_uris = []
                 playlist_uris = []
                 for uri in args[1:]:
-                    try:
-                        uri = GLib.filename_to_uri(uri)
-                    except:
-                        pass
-                    f = Gio.File.new_for_uri(uri)
-                    if not f.query_exists():
-                        uri = GLib.filename_to_uri(
-                            "%s/%s" % (GLib.get_current_dir(), uri))
+                    parsed = urlparse(uri)
+                    if parsed.scheme not in ["http", "https"]:
+                        try:
+                            uri = GLib.filename_to_uri(uri)
+                        except:
+                            pass
                         f = Gio.File.new_for_uri(uri)
-                    if is_audio(f):
-                        audio_uris.append(uri)
-                    elif is_pls(f):
+                        # Try ./filename
+                        if not f.query_exists():
+                            uri = GLib.filename_to_uri(
+                                "%s/%s" % (GLib.get_current_dir(), uri))
+                            print(uri)
+                            f = Gio.File.new_for_uri(uri)
+                    file_type = get_file_type(uri)
+                    if file_type == FileType.PLS:
                         playlist_uris.append(uri)
                     else:
-                        info = f.query_info(Gio.FILE_ATTRIBUTE_STANDARD_TYPE,
-                                            Gio.FileQueryInfoFlags.NONE,
-                                            None)
-                        if info.get_file_type() == Gio.FileType.DIRECTORY:
-                            audio_uris.append(uri)
+                        audio_uris.append(uri)
                 if playlist_uris:
                     self.__parse_uris(playlist_uris, audio_uris)
                 else:
@@ -600,22 +517,9 @@ class Application(Gtk.Application, ApplicationActions):
             @param uris as ([str], [str])
         """
         if playlist_uris:
-            self.__parse_uri(playlist_uris, audio_uris)
+            self.__parse_uris(playlist_uris, audio_uris)
         else:
-            to_scan_uris = []
-            for uri in audio_uris:
-                parsed = urlparse(uri)
-                if parsed.scheme in ["http", "https"]:
-                    from lollypop.objects_radio import Radio
-                    radio = Radio(Type.RADIOS)
-                    radio.set_name(uri)
-                    radio.set_uri(uri)
-                    self.player.load(radio)
-                    break
-                    # Lollypop does not support radio playlists
-                else:
-                    to_scan_uris.append(uri)
-            self.scanner.update(ScanType.EXTERNAL, to_scan_uris)
+            self.scanner.update(ScanType.EXTERNAL, audio_uris)
 
     def __on_entry_parsed(self, parser, uri, metadata, audio_uris):
         """
